# 计算机登录用户: jk
# 系统日期: 2023/5/17 9:44
# 项目名称: async_ccdt
# 开发者: zhanyong
from ..dataset.base_labelme import *
from ..dataset.base_coco import *
from ..dataset.utils import *
from ..dataset.base_voc import *
from ..dataset.base_csv import *



# 计算机登录用户: jk
# 系统日期: 2024/8/3 10:43
# 项目名称: chipeak_cv_data_tool
# 开发者: zhanyong
from pathlib import Path
from ccdt.dataset import *
import pandas as pd
import os
import shutil


class BaseCsv(BaseLabelme):
    def __init__(self, *args, **kwargs):
        self.output_csv = None
        self.output_dir = None
        # 在这里定义labelme数据结构格式初始化
        super(BaseCsv, self).__init__(*args, **kwargs)

    def labelme2csv(self):
        """
        labelme转csv
        """
        data = []
        for dataset in self.datasets:
            image_path = os.path.relpath(os.path.join(dataset.get('image_dir'), dataset.get('image_file'))).replace('\\', '/')
            image_width = dataset.get('image_width')
            image_height = dataset.get('image_height')
            stem = Path(dataset.get('input_dir')).stem
            self.output_csv = os.path.join(dataset.get('output_dir'), stem + '.csv')
            self.output_dir = os.path.join(dataset.get('output_dir'), dataset.get('image_dir'), stem)
            os.makedirs(self.output_dir, exist_ok=True)
            shutil.copy(dataset.get('full_path'), self.output_dir)
            if dataset.get('labelme_info') is not None:
                for shape in dataset.get('labelme_info').get('shapes'):
                    label = shape.get('label', '')
                    points = shape.get('points', [])
                    xmin = int(min(point[0] for point in points))
                    ymin = int(min(point[1] for point in points))
                    xmax = int(max(point[0] for point in points))
                    ymax = int(max(point[1] for point in points))
                    data.append({
                        'image_path': image_path,
                        'width': image_width,
                        'height': image_height,
                        'label': label,
                        'xmin': xmin,
                        'ymin': ymin,
                        'xmax': xmax,
                        'ymax': ymax
                    })
        # 将数据写入 CSV
        self.save_csv_image_file(data, self.output_csv)

    @staticmethod
    def save_csv_image_file(data_df, output_csv):
        try:
            df = pd.DataFrame(data_df)
            df.to_csv(output_csv, index=False)
            print(f"CSV file has been saved to {output_csv}")
        except Exception as e:
            print(e)



# 计算机登录用户: jk
# 系统日期: 2024/6/25 17:00
# 项目名称: chipeak_cv_data_tool
# 开发者: zhanyong
import os
import shutil
from ccdt.dataset import *
import xml.etree.ElementTree as ET
from pathlib import Path

# labelme转voc
class BaseXml(BaseLabelme):
    def __init__(self, *args, **kwargs):
        # 在这里定义labelme数据结构格式初始化
        super(BaseXml, self).__init__(*args, **kwargs)

    def labelme2xml(self):
        """
        labelme转xml数据集，即voc
        """
        for dataset in self.datasets:
            annotation = ET.Element('annotation')
            ET.SubElement(annotation, 'folder').text = ''  # 保障相对路径可以打开，文件夹写空，绝对路径需要指定为dataset.get('image_dir')
            ET.SubElement(annotation, 'filename').text = dataset.get('image_file')
            ET.SubElement(annotation, 'path').text = dataset.get('labelme_file')

            source = ET.SubElement(annotation, 'source')
            ET.SubElement(source, 'database').text = 'chipeak.com'

            size = ET.SubElement(annotation, 'size')
            ET.SubElement(size, 'width').text = str(dataset.get('image_width'))
            ET.SubElement(size, 'height').text = str(dataset.get('image_height'))
            ET.SubElement(size, 'depth').text = '3'  # Assuming RGB image

            ET.SubElement(annotation, 'segmented').text = '0'
            if dataset.get('labelme_info') is not None:
                for shape in dataset.get('labelme_info').get('shapes'):
                    label = shape['label']
                    points = shape['points']
                    xmin = int(min(point[0] for point in points))
                    ymin = int(min(point[1] for point in points))
                    xmax = int(max(point[0] for point in points))
                    ymax = int(max(point[1] for point in points))
                    obj = ET.SubElement(annotation, 'object')
                    ET.SubElement(obj, 'name').text = label
                    ET.SubElement(obj, 'pose').text = 'Unspecified'
                    ET.SubElement(obj, 'truncated').text = '0'
                    ET.SubElement(obj, 'difficult').text = '0'
                    bndbox = ET.SubElement(obj, 'bndbox')
                    ET.SubElement(bndbox, 'xmin').text = str(xmin)
                    ET.SubElement(bndbox, 'ymin').text = str(ymin)
                    ET.SubElement(bndbox, 'xmax').text = str(xmax)
                    ET.SubElement(bndbox, 'ymax').text = str(ymax)
                # Save XML file
                xml_str = ET.tostring(annotation, encoding='unicode')  # 使用 ET.tostring(annotation, encoding='unicode') 将 XML 树转换为字符串，这样可以避免自动生成的 XML 声明。
                self.save_xml_image_file(xml_str, dataset)

    @staticmethod
    def save_xml_image_file(xml_str, dataset):
        try:
            xml_filename = Path(dataset.get('image_file')).stem + '.xml'
            xml_path = os.path.join(dataset.get('output_dir'), dataset.get('image_dir'), xml_filename)
            image_dir = os.path.join(dataset.get('output_dir'), dataset.get('image_dir'))
            os.makedirs(image_dir, exist_ok=True)
            # 使用 with open(xml_path, 'w', encoding='utf-8') as f 手动写入 XML 字符串到文件。
            with open(xml_path, 'w', encoding='utf-8') as f:
                f.write(xml_str)
            # tree.write(xml_path, encoding='utf-8', xml_declaration=True)
            shutil.copy(dataset.get('full_path'), image_dir)
        except Exception as e:
            print(e)



# 计算机登录用户: jk
# 系统日期: 2023/7/31 18:06
# 项目名称: chipeak_cv_data_tool
# 开发者: zhanyong
import os
from pathlib import Path
from ccdt.dataset import *
import xml.etree.ElementTree as ET

# voc转labelme
class BaseVoc(BaseLabelme):
    def __init__(self, *args, **kwargs):
        self.voc_xml_path = args[1]  # 获取voc数据集对应的xml目录路径
        # 在这里定义labelme数据结构格式初始化
        super(BaseVoc, self).__init__(*args, **kwargs)

    def voc2labelme(self):
        """
        voc数据集转labelme数据集
        """
        voc_to_labelme = list()
        # 每循环一次，拼接对应的xml文件路径，直接读取xml文件，然后转换成labelme_info
        for dataset in self.datasets:
            image_file_name = dataset.get('image_file')
            obj_path = Path(image_file_name)
            xml_name = obj_path.stem + '.xml'
            voc_xml_path = os.path.join(self.voc_xml_path, xml_name)
            labelme_file = obj_path.stem + '.json'
            dataset.update({'image_dir': '00.images'})
            dataset.update({'labelme_file': labelme_file})
            relative_path = os.path.join('../', '00.images', image_file_name)
            labelme_info = self.analysis_xml(voc_xml_path)
            if labelme_info.get('shapes'):
                dataset.update({'background': True})
                labelme_info.update({'imagePath': relative_path})
                labelme_info.update({'imageHeight': dataset.get('image_height')})
                labelme_info.update({'imageWidth': dataset.get('image_width')})
                dataset.update({'relative_path': relative_path})
                dataset.update({'labelme_info': labelme_info})
            else:
                xml_path = dataset.get('full_path')
                print(f'xml标注对象为空f{xml_path}')
                dataset.update({'background': False})
            voc_to_labelme.append(dataset)
        self.save_labelme(voc_to_labelme, self.output_dir, None)  # self.output_dir为空字符串也是可以的

    @staticmethod
    def analysis_xml(xml_path):
        # 定义labelme数据结构
        labelme_data = dict(
            version='4.5.14',
            flags={},
            shapes=[],
            imagePath=None,
            imageData=None,
            imageHeight=None,
            imageWidth=None
        )
        # 解析XML文件
        tree = ET.parse(xml_path)
        root = tree.getroot()
        # 查找所有的object节点
        object_nodes = root.iter('object')
        # 遍历object节点
        for obj in object_nodes:
            # 根据需要提取所需的信息
            name = obj.find('name').text
            # pose = obj.find('pose').text
            bndbox = obj.find('bndbox')
            xmin = float(bndbox.find('xmin').text)  # 把字符串转float类型
            ymin = float(bndbox.find('ymin').text)
            xmax = float(bndbox.find('xmax').text)
            ymax = float(bndbox.find('ymax').text)
            points = [[xmin, ymin], [xmax, ymax]]  # 坐标点计算，目前使用左上角的点和右下角的点计算
            shape = {"label": name, "points": points, "group_id": None, "shape_type": "rectangle", "flags": {}, 'text': None}
            labelme_data.get('shapes').append(shape)
        return labelme_data



# 计算机登录用户: jk
# 系统日期: 2024/6/25 17:10
# 项目名称: chipeak_cv_data_tool
# 开发者: zhanyong

import os
import shutil
from pathlib import Path
from tqdm import *
from ccdt.dataset import *
import subprocess
import json


# labelme转yolo
class BaseYolo(BaseLabelme):
    def __init__(self, *args, **kwargs):
        self.yolo_path = args[1].output_dir  # 获取yolo数据集输出目录路径
        self.txt_path = args[1].input_datasets  # 获取yolo的txt文件目录路径
        # 在这里定义labelme数据结构格式初始化
        super(BaseYolo, self).__init__(*args, **kwargs)

    def labelme2yolo(self):
        """
        labelme转yolo数据集
        """
        print("labelme转yolo开始*********************")
        for dataset in tqdm(self.datasets):
            output_dir = dataset.get('output_dir')
            input_dir = dataset.get('input_dir')
            image_file_stem = Path(dataset.get('image_file')).stem  # 获取文件前缀
            class_names = []  # 类别和坐标
            # yolo_txt_dir_path = os.path.join(output_dir, dataset.get('labelme_dir'), "labels")
            # yolo_image_dir_path = os.path.join(output_dir, dataset.get('image_dir'), "images")
            yolo_txt_dir_path = os.path.join(output_dir, "labels")  # 默认开源工程部能够嵌套目录
            yolo_image_dir_path = os.path.join(output_dir, "images")
            yolo_txt_file_path = os.path.join(yolo_txt_dir_path, image_file_stem + ".txt")  # 替换 .replace('\\', '/')
            os.makedirs(yolo_txt_dir_path, exist_ok=True)  # 创建yolo格式标注数据目录
            os.makedirs(yolo_image_dir_path, exist_ok=True)  # 创建yolo格式图片目录
            # # 定义用户和组
            # user = 'scanner'
            # group = 'sambashare'
            # # 修改目录权限
            # # os.chmod(yolo_txt_file_path, 0o775)
            # os.chmod(yolo_image_dir_path, 0o775)
            # # 修改目录所有者和组
            # # subprocess.run(['chown', f'{user}:{group}', yolo_txt_file_path], check=True)
            # subprocess.run(['chown', f'{user}:{group}', yolo_image_dir_path], check=True)
            width = dataset.get('image_width')
            height = dataset.get('image_height')
            if dataset.get('background') is True:
                for index, shape in enumerate(dataset.get('labelme_info').get('shapes')):
                    # 获取边界框坐标
                    points = shape["points"]
                    xmin = min(points[0][0], points[1][0])
                    ymin = min(points[0][1], points[1][1])
                    xmax = max(points[0][0], points[1][0])
                    ymax = max(points[0][1], points[1][1])
                    # 转换为 YOLO 格式 (class, x_center, y_center, width, height)
                    x_center = (xmin + xmax) / 2.0 / width
                    y_center = (ymin + ymax) / 2.0 / height
                    bbox_width = (xmax - xmin) / width
                    bbox_height = (ymax - ymin) / height
                    # print(index, x_center, y_center, bbox_width, bbox_height)
                    class_names.append([index, x_center, y_center, bbox_width, bbox_height])
            # 检查目录写入权限
            # if self.check_write_permission(yolo_txt_file_path):
            #     print(f"目录 {yolo_txt_file_path} 具有写入权限。")
            # else:
            #     print(f"目录 {yolo_txt_file_path} 不具有写入权限。")
            # 保存类别文件
            with open(yolo_txt_file_path, "w") as class_f:
                for class_name in class_names:
                    # 将每个 class_name=[0, 0.5012201365187713, 0.5480937499999999, 0.9975597269624574, 0.8811562500000001] 中的值以 YOLO 格式写入文件
                    class_f.write(" ".join(map(str, class_name)) + "\n")
            # 拷贝图像
            shutil.copy(dataset.get('full_path'), yolo_image_dir_path)

    def yolo2labelme(self):
        # 将单引号替换为双引号，并将反斜杠转义
        json_str = self.txt_path.replace("'", '"').replace("\\", "\\\\")
        # 解析 JSON 字符串
        json_data = json.loads(json_str)
        # 获取 input_txt_dir 的值
        input_txt_dir = json_data[0]['input_txt_dir']
        for dataset in tqdm(self.datasets):
            relative_path = dataset.get("relative_path").replace("images", "00.images")
            txt_file = Path(dataset.get("image_file")).stem + ".txt"
            label_file = os.path.join(input_txt_dir, txt_file).replace("\\", "/")
            labelme_data = dict(
                version='4.5.9',
                flags={},
                shapes=[],
                imagePath=None,
                imageData=None,
                imageHeight=None,
                imageWidth=None,
                md5Value=None
            )
            shapes = []
            with open(label_file, 'r') as file:
                for line in file:
                    # 计算左上角和右下角的坐标
                    class_id, x_min, y_min, x_max, y_max = self.yolo_to_bbox(line.strip(), dataset.get("image_width"), dataset.get("image_height"))
                    points = [[x_min, y_min], [x_max, y_max]]
                    # 只处理带矩形框，并转labelme
                    shape = {"label": str(class_id), "points": points, "group_id": None, "shape_type": "rectangle", "flags": {}, 'text': None}
                    shapes.append(shape)
            # 'lines' 现在是一个包含处理后的每一行的数组
            # print(shapes)
            labelme_data.update({'shapes': shapes})
            labelme_data.update({'imagePath': relative_path})
            labelme_data.update({'imageWidth': dataset.get("image_width")})
            labelme_data.update({'imageHeight': dataset.get("image_height")})
            labelme_data.update({'md5Value': dataset.get("md5_value")})
            dataset.update({'labelme_info': labelme_data})
            dataset.update({'image_dir': '00.images'})
            dataset.update({'background': True})
        self.save_labelme(self.datasets, self.yolo_path, None)

    @staticmethod
    def check_write_permission(directory):
        return os.access(directory, os.W_OK)

    @staticmethod
    def yolo_to_bbox(yolo_data_str, image_width, image_height):
        """
        # YOLO格式中的标注数据通常是相对于图像尺寸的归一化数据，格式如下：class_id center_x center_y width height
        @param yolo_data_str:
        @param image_width:
        @param image_height:
        @return:
        """
        # 将字符串拆分为列表，并转换数值类型
        yolo_data = yolo_data_str.split()
        yolo_data = [int(yolo_data[0])] + [float(i) for i in yolo_data[1:]]
        # 解析输入的 YOLO 数据
        class_id, center_x, center_y, width, height = yolo_data
        # 计算左上角和右下角的坐标
        x_min = (center_x - width / 2) * image_width
        y_min = (center_y - height / 2) * image_height
        x_max = (center_x + width / 2) * image_width
        y_max = (center_y + height / 2) * image_height
        return class_id, x_min, y_min, x_max, y_max
